@@ -1,7 +1,7 @@
 //! A generic [`Value`] which supports collation
 
 use std::cmp::Ordering;
-use std::convert::TryFrom;
+use std::convert::{TryFrom, TryInto};
 use std::fmt;
 use std::iter::FromIterator;
 use std::str::FromStr;
@@ -16,6 +16,7 @@ use log::debug;
 use safecast::{CastFrom, CastInto, TryCastFrom, TryCastInto};
 use serde::de::{Deserialize, Deserializer, Error as SerdeError};
 use serde::ser::{Serialize, SerializeMap, Serializer};
+use uuid::Uuid;
 
 use tc_error::*;
 use tcgeneric::*;
@@ -262,6 +263,7 @@ impl From<NumberType> for ValueType {
 impl TryCastFrom<Value> for ValueType {
     fn can_cast_from(value: &Value) -> bool {
         match value {
+            Value::Link(l) if l.host().is_none() => Self::from_path(l.path()).is_some(),
             Value::String(s) => {
                 if let Ok(path) = TCPathBuf::from_str(s) {
                     Self::from_path(&path).is_some()
@@ -298,10 +300,20 @@ impl de::FromStream for ValueType {
     type Context = ();
 
     async fn from_stream<D: de::Decoder>(_: (), decoder: &mut D) -> Result<Self, D::Error> {
+        debug!("ValueType::from_stream");
         decoder.decode_any(ValueTypeVisitor).await
     }
 }
 
+impl<'en> en::IntoStream<'en> for ValueType {
+    fn into_stream<E: en::Encoder<'en>>(self, encoder: E) -> Result<E::Ok, E::Error> {
+        use en::EncodeMap;
+        let mut map = encoder.encode_map(Some(1))?;
+        map.encode_entry(self.path(), Map::<Value>::default())?;
+        map.end()
+    }
+}
+
 impl fmt::Display for ValueType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -354,7 +366,7 @@ impl DestreamVisitor for ValueTypeVisitor {
                     Err(DestreamError::invalid_value(map, "empty map"))
                 }
             } else {
-                Err(DestreamError::custom("invalid Value type"))
+                Err(de::Error::custom("invalid Value class"))
             }
         } else {
             Err(DestreamError::invalid_length(0, Self::expecting()))
@@ -375,6 +387,7 @@ pub enum Value {
 }
 
 impl Value {
+    /// Return a [`TCError`] if this `Value` is not none.
     #[inline]
     pub fn expect_none(&self) -> TCResult<()> {
         if self.is_none() {
@@ -384,6 +397,7 @@ impl Value {
         }
     }
 
+    /// Return `true` if this `Value` is a default [`Link`], `Value::None`, or empty [`Tuple`].
     pub fn is_none(&self) -> bool {
         match self {
             Self::Link(link) => link.host().is_none() && link.path() == &TCPathBuf::default(),
@@ -393,10 +407,19 @@ impl Value {
         }
     }
 
+    /// Return `true` if this value is *not* a default [`Link`], `Value::None`, or empty [`Tuple`].
     pub fn is_some(&self) -> bool {
         !self.is_none()
     }
 
+    /// Return `true` if this `Value` variant is a [`Tuple`].
+    pub fn is_tuple(&self) -> bool {
+        match self {
+            Self::Tuple(_) => true,
+            _ => false,
+        }
+    }
+
     pub fn into_type(self, class: ValueType) -> Option<Self> {
         if self.class() == class {
             return Some(self);
@@ -662,12 +685,6 @@ impl TryFrom<Value> for Tuple<Value> {
     }
 }
 
-impl CastFrom<Value> for String {
-    fn cast_from(value: Value) -> String {
-        value.to_string()
-    }
-}
-
 impl TryCastFrom<Value> for Bytes {
     fn can_cast_from(value: &Value) -> bool {
         match value {
@@ -748,7 +765,52 @@ impl TryCastFrom<Value> for Number {
                 }
             }
             Value::Tuple(_) => None,
-            Value::String(s) => f64::from_str(&s).map(Float::F64).map(Self::Float).ok(),
+            Value::String(s) => Number::from_str(&s).ok(),
+        }
+    }
+}
+
+impl TryCastFrom<Value> for String {
+    fn can_cast_from(value: &Value) -> bool {
+        match value {
+            Value::Link(_) => true,
+            Value::Id(_) => true,
+            Value::Number(_) => true,
+            Value::String(_) => true,
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(value: Value) -> Option<Self> {
+        match value {
+            Value::Link(link) => Some(link.to_string()),
+            Value::Id(id) => Some(id.to_string()),
+            Value::Number(n) => Some(n.to_string()),
+            Value::String(s) => Some(s),
+            _ => None,
+        }
+    }
+}
+
+impl TryCastFrom<Value> for Uuid {
+    fn can_cast_from(value: &Value) -> bool {
+        match value {
+            Value::Bytes(bytes) => bytes.len() == 16,
+            Value::Id(id) => Uuid::from_str(id.as_str()).is_ok(),
+            Value::String(s) => Uuid::from_str(s).is_ok(),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(value: Value) -> Option<Self> {
+        match value {
+            Value::Bytes(bytes) if bytes.len() == 16 => {
+                let bytes = bytes.to_vec().try_into().expect("16-byte UUID");
+                Some(Uuid::from_bytes(bytes))
+            }
+            Value::Id(id) => id.as_str().parse().ok(),
+            Value::String(s) => s.parse().ok(),
+            _ => None,
         }
     }
 }
@@ -848,12 +910,6 @@ impl<T1: TryCastFrom<Value>, T2: TryCastFrom<Value>> TryCastFrom<Value> for (T1,
     }
 
     fn opt_cast_from(value: Value) -> Option<Self> {
-        debug!(
-            "cast from {} into {}?",
-            value,
-            std::any::type_name::<Self>()
-        );
-
         match value {
             Value::Tuple(tuple) => Self::opt_cast_from(tuple),
             _ => None,
@@ -919,14 +975,14 @@ impl fmt::Debug for Value {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Self::Bytes(bytes) => write!(f, "({} bytes)", bytes.len()),
-            Self::Id(id) => write!(f, "{} {:?}", ValueType::Id, id.as_str()),
-            Self::Link(link) => write!(f, "{} {:?}", ValueType::Link, link),
+            Self::Id(id) => write!(f, "{}: {:?}", ValueType::Id, id.as_str()),
+            Self::Link(link) => write!(f, "{}: {:?}", ValueType::Link, link),
             Self::None => f.write_str("None"),
             Self::Number(n) => fmt::Debug::fmt(n, f),
-            Self::String(s) => write!(f, "{}, {}", ValueType::String, s),
+            Self::String(s) => write!(f, "{}: {}", ValueType::String, s),
             Self::Tuple(t) => write!(
                 f,
-                "{} ({})",
+                "{}: ({})",
                 ValueType::Tuple,
                 t.iter()
                     .map(|v| format!("{:?}", v))
@@ -1022,6 +1078,14 @@ impl ValueVisitor {
     ) -> Result<Value, A::Error> {
         use ValueType as VT;
 
+        if let Ok(map) = map.next_value::<Map<Value>>(()).await {
+            return if map.is_empty() {
+                Ok(Value::Link(class.path().into()))
+            } else {
+                Err(de::Error::invalid_value(map, "a Value class"))
+            };
+        }
+
         return match class {
             VT::Bytes => {
                 let bytes = map.next_value(()).await?;
